/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /*
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/
});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function() {
    /*
     *   Add custom actions for menu items using the following api:
     *
     *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
     *
     *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
     *   param runable - function that performs the custom action.
     *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
     *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
     */

    // For example,
    // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

    // Alternatively, to programmatically disable a menu action at any time use:
    // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

    gc.nav.registerAction('show_introduction', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_motor_control', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('load_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            gc.File.browseAndLoad(null, {
                bin: false
            }, function(contents, properties) {
                var regData = null;
                try {
                    regData = JSON.parse(contents);
                    if (regData) {
                        templateObj.setRegisterData(regData);
                        templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
                    }
                } catch (ex) {
                    templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
                }

            });
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    var saveText = function(fname, text, mime) {
        var blob = new Blob([text], {
            type: mime
        });
        saveAs(blob, fname);
        return false;
    }
    gc.nav.registerAction('save_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: saved to " + result.localPath);
                    }

                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('save_registers_as', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers As: saved to " + result.localPath);
                    }
                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_log_pane', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_eventlog_view.openView();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_scripting_dialog', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.scripting_dialog.removeAttribute('hidden');
            templateObj.$.scripting_dialog.open();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var templateObj;
var registerModel;


// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
    templateObj.addEventListener('dom-change', function() {
        if (initComplete) return;
        this.async(function() {
            initComplete = true;
            console.log("Application template has been stamped.");
            templateObj.$.ti_widget_toast.hideToast();
            templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
            registerModel = Polymer.dom(document).querySelector('ti-model-register');
            if (registerModel) {
                registerModel.addEventListener("usbHidModelReceivingData", function (event) {
                    templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
                });
            } else {
                console.log("no ti-model-register component found in this application!")
            }

            // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
            // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

            // open the eventlog viewer when the user clicks on the book icon in the status bar:
            templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
            templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function(event) {
                templateObj.$.ti_widget_eventlog_view.toggleView();
            });


            // Expand vtabcontainer nav bar when user clicks on menu icon or 'Menu' label
            templateObj.toggleMenu = function(event){
                templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
            };
            templateObj.$.ti_widget_icon_button_menu.addEventListener('click',templateObj.toggleMenu);
            templateObj.$.ti_widget_label_menu.addEventListener('click',templateObj.toggleMenu);

            if (templateObj.$.ti_widget_register_page) {
                // get the active device name from the register page
                templateObj.$.ti_widget_menubar.productName = templateObj.$.ti_widget_register_page.deviceName;

                // called when the user clicks the Write Register button:
                templateObj.$.ti_widget_register_page.addEventListener("write_reg", function (event) {
					console.log("Uttam Write Reg");
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user clicks the Read Register button:
                templateObj.$.ti_widget_register_page.addEventListener("read_reg", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user clicks the Read All Registers button:
                templateObj.$.ti_widget_register_page.addEventListener("read_regs", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user changes the immediate / deferred mode
                templateObj.$.ti_widget_register_page.addEventListener("update_mode_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
                }.bind(this));

                // called when the user changes the auto read refresh rate
                templateObj.$.ti_widget_register_page.addEventListener("auto_read_config_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
                }.bind(this));
            }

            var usbHidService = gc.services['ti-service-usbhid'];
            if (usbHidService) {
                usbHidService.addEventListener('usbHidError', function(event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
                })
            }

            templateObj.getRegisterData = function() {
                var regData = {
                    signature: "oneui-register-data",
                    data: []
                }
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var i = 0; i < regBlocks.length; i++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[i].registers.length; r++) {
                        var reg = regBlocks[i].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        var item = {
                            idx: r,
                            id: regName,
                            value: reg.value
                        }
                        registersData.push(item);
                    }
                    regData.data.push(registersData);
                }
                var strJson = JSON.stringify(regData);
                return strJson;
            };

            templateObj.setRegisterData = function(strJson) {
                var regData = JSON.parse(strJson);
                var ok = true;
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var b = 0; b < regBlocks.length; b++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[b].registers.length; r++) {
                        var reg = regBlocks[b].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        item = regData.data[b][r];
                        if (item.id === regName) {
                            reg.value = item.value;
                            var intValue = gc.utils.getValueFromHexString(""+reg.value);
                            templateObj.$.ti_widget_register_page.writeValueToTargetRegister(intValue, null, b, r);
                        } else {
                            var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
                            templateObj.logMsgAndNotify("warning", msg, false, true);
                        }
                    }

                }
            };

            templateObj.showToastMsg = function(msgType, msg, durationInSeconds) {
                templateObj.$.ti_widget_toast.hideToast();
                templateObj.$.ti_widget_toast.message = msg;
                switch (msgType.toLowerCase()) {
                    case "error":
                        templateObj.$.ti_widget_toast.iconName = "error";
                        templateObj.$.ti_widget_toast.backgroundColor = "red";
                        break;
                    case "warning":
                        templateObj.$.ti_widget_toast.iconName = "warning";
                        templateObj.$.ti_widget_toast.backgroundColor = "orange";
                        break;
                    default:
                        templateObj.$.ti_widget_toast.iconName = "check";
                        templateObj.$.ti_widget_toast.backgroundColor = "teal";
                        break;
                }
                templateObj.$.ti_widget_toast.duration = +durationInSeconds;
                templateObj.$.ti_widget_toast.showToast();
                console.log("toast msg [" + msgType + "]: " + msg);
            };

            templateObj.logMsgAndNotify = function(msgType, msg, showInStatusBar, showInToast) {
                var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
                if (showInStatusBar) {
                    var statusBar = gc.services['ti-widget-statusbar'];
                    if (statusBar) {
                        statusBar.statusString3 = msg;
                        statusBar.setIcon(iconName);
                    }
                }
                if (showInToast) {
                    templateObj.showToastMsg(msgType, msg, 4);
                }
                templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
            };
            templateObj.$.ti_widget_label_learnmore.addEventListener("click",function(event){
                 templateObj.$.ti_widget_label_productname.fontColor="#444";
                 templateObj.$.ti_widget_label_productdesc.fontColor="#444";
                 templateObj.$.ti_widget_label_learnmore.setAttribute('hidden',true);
                 templateObj.$.ti_widget_label_showless.removeAttribute('hidden');
                 templateObj.$.ti_widget_vtab_home.style.backgroundColor = "white";
                 templateObj.$.ti_widget_tabcontainer.selectedIndex = 1;
            });
            templateObj.$.ti_widget_label_showless.addEventListener("click",function(event){
                 templateObj.$.ti_widget_label_productname.fontColor="white";
                 templateObj.$.ti_widget_label_productdesc.fontColor="white";
                 templateObj.$.ti_widget_label_learnmore.removeAttribute('hidden');
                 templateObj.$.ti_widget_label_showless.setAttribute('hidden',true);
                 templateObj.$.ti_widget_vtab_home.style.backgroundColor = "rgb(20, 140, 156)";
                 templateObj.$.ti_widget_tabcontainer.selectedIndex = 0;
            });
	    if (templateObj.$.ti_widget_button_connect) {
                templateObj.$.ti_widget_button_connect.addEventListener("click",function(event){
                    if (gc.connectionManager){
                        gc.connectionManager.connect();
                    }
                });
	    }
		
		//Added by Akhilesh K for DAC53204EVM
		// The register model reads in the .json file when it starts up, so in order to get 
        // access to this information, we need to wait for the register model to send a registerDefsChanged
        // event that says it has updated its register model:
            
        registerModel = Polymer.dom(document).querySelector('ti-model-register');
			
// DAC Gain Configurations Channel 0
		registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_vout_cmp_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_gain_dac_0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_gain_dac_0.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_gain_dac_0.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
			
// DAC Gain Configurations Channel 1
		registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_vout_cmp_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_gain_dac_1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_gain_dac_1.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_gain_dac_1.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
			
			
// DAC Gain Configurations Channel 2
		registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_vout_cmp_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_gain_dac_2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_gain_dac_2.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_gain_dac_2.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
			
// DAC Gain Configurations Channel 3
		registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_vout_cmp_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_gain_dac_3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_gain_dac_3.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_gain_dac_3.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
	
// IOUT RANGE Configurations
// Channel 0
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_iout_misc_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "iout_range_dac0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_iout_range_dac_0.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_iout_range_dac_0.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Channel 1
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_iout_misc_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "iout_range_dac1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_iout_range_dac_1.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_iout_range_dac_1.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Channel 2
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_iout_misc_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "iout_range_dac2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_iout_range_dac_2.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_iout_range_dac_2.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Channel 3
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_iout_misc_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "iout_range_dac3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_iout_range_dac_3.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_iout_range_dac_3.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// POWER DOWN Configurations for Voltage Modes
// Channel 0
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "common_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_pdn_dac0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_0.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_0.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Channel 1
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "common_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_pdn_dac1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_1.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_1.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Channel 2
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "common_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_pdn_dac2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_2.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_2.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Channel 3
        registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "common_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "vout_pdn_dac3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_3.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_vout_pdn_dac_3.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Code Step for DAC0
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_0.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_0.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Code Step for DAC1
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_1.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_1.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});	
// Code Step for DAC2
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_2.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_2.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});			
// Code Step for DAC3
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_3.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_3.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});	

// slew rate for DAC0
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_0.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_0.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});			
// slew rate for DAC1
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_1.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_1.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});						
// slew rate for DAC2
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_2.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_2.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});						
// slew rate for DAC3
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_3.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_3.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});						
// Waveform Generation for DAC0
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "func_config_0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_func_config_0.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_func_config_0.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});						
// Waveform Generation for DAC1
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "func_config_1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_func_config_1.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_func_config_1.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});				
// Waveform Generation for DAC2
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "func_config_2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_func_config_2.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_func_config_2.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});				
// Waveform Generation for DAC3
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "func_config_3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_func_config_3.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_func_config_3.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});				
			
// Phase selection for DAC0
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "phase_sel_0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_phase_sel_0.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_phase_sel_0.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});	
// Phase selection for DAC1
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "phase_sel_1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_phase_sel_1.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_phase_sel_1.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Phase selection for DAC2
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "phase_sel_2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_phase_sel_2.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_phase_sel_2.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Phase selection for DAC3
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "phase_sel_3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_phase_sel_3.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_phase_sel_3.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
			
// Slew Rate again for DAC Function Generation section
// slew rate for DAC0
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_4.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_4.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});			
// slew rate for DAC1
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_5.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_5.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});						
// slew rate for DAC2
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_6.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_6.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});						
// slew rate for DAC3
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "slew_rate_3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_slew_rate_7.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_slew_rate_7.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});						
// Code step for function Generation section
// Code Step for DAC0
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_0_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_0"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_4.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_4.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});
// Code Step for DAC1
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_1_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_1"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_5.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_5.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});	
// Code Step for DAC2
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_2_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_2"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_6.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_6.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});			
// Code Step for DAC3
         registerModel.getRegisterViewInformation().then(function(deviceObj) {
				// filter the register blocks based on id
				const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "dacx3204_registers"));
				if (regBlockArray.length > 0) {
					const regBlock = regBlockArray[0];
					// filter the registers from that register block based on id
					const regArray = regBlock.registers.filter((reg) => (reg.id === "dac_3_func_config"));
					if (regArray.length > 0) {
						const reg = regArray[0];
						// filter the bitfields from that register based on id
						bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "code_step_3"));
						if (bitfieldArray.length > 0) {
							const bitfield = bitfieldArray[0];
							// use the options for the bitfield to create the labels and values for the droplist
							const widgetOptions = bitfield.widget.options;
							const labels = widgetOptions.map((option) => (option.display));
							templateObj.$.ti_widget_droplist_code_step_7.labels = labels.join("|");
							const values = widgetOptions.map((option) => (option.value));
							templateObj.$.ti_widget_droplist_code_step_7.values = values.join(",");
							// the widget's binding to the register bitfield controls what the selected value and label is
						}
					}
				}
			});	
			
        }, 1);
    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init);
}